import type { MergeExclusive, NonEmptyObject } from 'type-fest';
import type { IsMultipleKeys } from '../lib/prelude.js';
import type { TSError } from '../lib/TSError.js';
import type { Schema } from '../Schema/__.js';
import type { Config, ReturnMode } from './Config.js';
import type { ResultSet } from './ResultSet/__.js';
import { SelectionSet } from './SelectionSet/__.js';
import type { DocumentObject } from './SelectionSet/toGraphQLDocumentString.js';
export type DocumentFn<$Config extends Config, $Index extends Schema.Index> = <$Document extends Document<$Index>>(document: ValidateDocumentOperationNames<NonEmptyObject<$Document>>) => {
    run: <$Name extends keyof $Document & string, $Params extends (IsMultipleKeys<$Document> extends true ? [name: $Name] : ([] | [name: $Name | undefined]))>(...params: $Params) => Promise<ReturnMode<$Config, ResultSet.Root<GetOperation<$Document[$Name]>, $Index, 'Query'>>>;
};
export declare const toDocumentExpression: (document: DocumentObject) => string;
export type Document<$Index extends Schema.Index> = {
    [name: string]: $Index['Root']['Query'] extends null ? {
        mutation: SelectionSet.Root<$Index, 'Mutation'>;
    } : $Index['Root']['Mutation'] extends null ? {
        query: SelectionSet.Root<$Index, 'Query'>;
    } : MergeExclusive<{
        query: SelectionSet.Root<$Index, 'Query'>;
    }, {
        mutation: SelectionSet.Root<$Index, 'Mutation'>;
    }>;
};
export type ValidateDocumentOperationNames<$Document> = string | number extends keyof $Document ? $Document : keyof {
    [K in keyof $Document & string as Schema.Named.NameParse<K> extends never ? K : never]: K;
} extends never ? $Document : TSError<'ValidateDocumentOperationNames', `One or more Invalid operation name in document: ${keyof {
    [K in keyof $Document & string as Schema.Named.NameParse<K> extends never ? K : never]: K;
}}`>;
type GetOperation<T extends {
    query: any;
} | {
    mutation: any;
}> = T extends {
    query: infer U;
} ? U : T extends {
    mutation: infer U;
} ? U : never;
export {};
//# sourceMappingURL=document.d.ts.map