import { buildSchema } from 'graphql';
import * as Path from 'node:path';
import { getTypeMapByKind } from '../../lib/graphql.js';
import { generateError } from './Error.js';
import { generateGlobal } from './global.js';
import { generateIndex } from './Index.js';
import { generateScalar } from './Scalar.js';
import { generateSchemaBuildtime } from './SchemaBuildtime.js';
import { generateRuntimeSchema } from './SchemaRuntime.js';
import { generateSelect } from './Select.js';
export const resolveOptions = (input) => {
    const errorTypeNamePattern = input.options?.errorTypeNamePattern ?? null;
    const schema = buildSchema(input.schemaSource);
    const typeMapByKind = getTypeMapByKind(schema);
    const errorObjects = errorTypeNamePattern
        ? Object.values(typeMapByKind.GraphQLObjectType).filter(_ => _.name.match(errorTypeNamePattern))
        : [];
    return {
        name: input.name ?? `default`,
        schema,
        error: {
            enabled: Boolean(errorTypeNamePattern),
            objects: errorObjects,
        },
        importPaths: {
            customScalarCodecs: input.importPaths?.customScalarCodecs ?? Path.join(process.cwd(), `customScalarCodecs.js`),
        },
        libraryPaths: {
            scalars: input.libraryPaths?.scalars ?? `graphql-request/alpha/schema/scalars`,
            schema: input.libraryPaths?.schema ?? `graphql-request/alpha/schema`,
        },
        typeMapByKind,
        options: {
            errorTypeNamePattern,
            customScalars: input.options?.customScalars ?? false,
            TSDoc: {
                noDocPolicy: input.options?.TSDoc?.noDocPolicy ?? `ignore`,
            },
        },
    };
};
export const generateCode = (input) => {
    const defaultDprintConfig = {
        quoteStyle: `preferSingle`,
        semiColons: `asi`,
    };
    const format = (source) => input.options?.formatter?.formatText(`memory.ts`, source, defaultDprintConfig) ?? source;
    const config = resolveOptions(input);
    return [
        generateGlobal,
        generateError,
        generateIndex,
        generateScalar,
        generateSchemaBuildtime,
        generateRuntimeSchema,
        generateSelect,
    ].map(_ => _(config)).map(code => ({
        ...code,
        code: format(code.code),
    }));
};
//# sourceMappingURL=generateCode.js.map